@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2007-2010  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2007-2019  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -45,9 +45,6 @@ refcount_ptr<Component> DummyComponent::Create(const ComponentCreateArgs& args)
 
 string DummyComponent::GetAttribute(const string& attributeName)
 {
-	if (attributeName == "stable")
-		return "yes";
-
 	if (attributeName == "description")
 		return "A dummy component, which does nothing.";
 
