@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2014  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2019  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -328,10 +328,21 @@ static int console_stdin_avail(int handle)
 }
 
 
+static int console_room_left_in_fifo(int handle)
+{
+	int roomLeftInFIFO = console_handles[handle].fifo_tail
+	    - console_handles[handle].fifo_head;
+	if (roomLeftInFIFO <= 0)
+		roomLeftInFIFO += CONSOLE_FIFO_LEN;
+
+	return roomLeftInFIFO;
+}
+
+
 /*
  *  console_charavail():
  *
- *  Returns 1 if a char is available in the fifo, 0 otherwise.
+ *  Returns the number of chararacters available in the fifo.
  */
 int console_charavail(int handle)
 {
@@ -342,9 +353,7 @@ int console_charavail(int handle)
 
 		// If adding more would lead to a full FIFO, then let's
 		// wait.
-		int roomLeftInFIFO = console_handles[handle].fifo_tail - console_handles[handle].fifo_head;
-		if (roomLeftInFIFO <= 0)
-			roomLeftInFIFO += CONSOLE_FIFO_LEN;
+		int roomLeftInFIFO = console_room_left_in_fifo(handle);
 		if (roomLeftInFIFO < (int)sizeof(ch) + 1)
 			break;
 
@@ -369,11 +378,7 @@ int console_charavail(int handle)
 		}
 	}
 
-	if (console_handles[handle].fifo_head ==
-	    console_handles[handle].fifo_tail)
-		return 0;
-
-	return 1;
+	return CONSOLE_FIFO_LEN - console_room_left_in_fifo(handle);
 }
 
 
