@@ -2,7 +2,7 @@
 #define	CPU_MIPS_H
 
 /*
- *  Copyright (C) 2003-2010  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2019  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -228,7 +228,8 @@ struct mips_cpu {
 	struct interrupt irq_compare;
 	struct timer	*timer;
 
-	int		rmw;		/*  Read-Modify-Write  */
+	/*  Read-Modify-Write (LL/SC):  */
+	int		rmw;		/*  1 = currently active  */
 	uint64_t	rmw_len;	/*  Length of rmw modification  */
 	uint64_t	rmw_addr;	/*  Address of rmw modification  */
 
